/**
 *  Created     :   2021-03-1
 *  Copyright (c)  CMCC Technologies, All rights reserved.
 */

#include "SimpleInput.h"
#include "utils/simple_log.h"
#include "utils/SimpleMath.h"
#include "utils/SimpleEventManager.h"
#include "utils/simple_utils.h"
#include <cassert>
#include <thread>

using namespace SIMPLE_MATH;
using namespace SIMPLE_EVENT;
using namespace SIMPLE_UTILS;

namespace SIMPLE_INPUT{
#define GAIN 0.1f
#define freq 512.0f
    static void input_main(SimpleInput* simpleInput){
        RT_LOGI("Start input.");
        assert(simpleInput);
        simpleInput->Update();
    }
    struct {
        volatile float w=1.0;
        volatile float x=0.0;
        volatile float y=0.0;
        volatile float z=0.0;
    } outQ;
    static float inverse_square(float h){
        float half = 0.5f * h;
        float y = h;
        long i = *(long*)&y;
        i = 0x5f3759df - (i>>1);
        y = *(float*)&i;
        y = y * (1.5f - (half * y * y));
        return y;
    }
    static SimplePose get_pose_from_imu(float gx, float gy, float gz,
                                     float ax, float ay, float az) {
        SimplePose simplePose;
        float recipNorm;
        float s0, s1, s2, s3;
        float qDot1, qDot2, qDot3, qDot4;
        float _2q0, _2q1, _2q2, _2q3, _4q0, _4q1, _4q2 ,_8q1, _8q2, q0q0, q1q1, q2q2, q3q3;
        qDot1 = 0.5f * (-outQ.x * gx - outQ.y * gy - outQ.z * gz);
        qDot2 = 0.5f * (outQ.w * gx + outQ.y * gz - outQ.z * gy);
        qDot3 = 0.5f * (outQ.w * gy - outQ.x * gz + outQ.z * gx);
        qDot4 = 0.5f * (outQ.w * gz + outQ.x * gy - outQ.y * gx);
        if(!((ax == 0.0f) && (ay == 0.0f) && (az == 0.0f))) {
            recipNorm = inverse_square(ax * ax + ay * ay + az * az);
            ax *= recipNorm;
            ay *= recipNorm;
            az *= recipNorm;
            _2q0 = 2.0f * outQ.w;
            _2q1 = 2.0f * outQ.x;
            _2q2 = 2.0f * outQ.y;
            _2q3 = 2.0f * outQ.z;
            _4q0 = 4.0f * outQ.w;
            _4q1 = 4.0f * outQ.x;
            _4q2 = 4.0f * outQ.y;
            _8q1 = 8.0f * outQ.x;
            _8q2 = 8.0f * outQ.y;
            q0q0 = outQ.w * outQ.w;
            q1q1 = outQ.x * outQ.x;
            q2q2 = outQ.y * outQ.y;
            q3q3 = outQ.z * outQ.z;
            s0 = _4q0 * q2q2 + _2q2 * ax + _4q0 * q1q1 - _2q1 * ay;
            s1 = _4q1 * q3q3 - _2q3 * ax + 4.0f * q0q0 * outQ.x - _2q0 * ay - _4q1 + _8q1 * q1q1 + _8q1 * q2q2 + _4q1 * az;
            s2 = 4.0f * q0q0 * outQ.y + _2q0 * ax + _4q2 * q3q3 - _2q3 * ay - _4q2 + _8q2 * q1q1 + _8q2 * q2q2 + _4q2 * az;
            s3 = 4.0f * q1q1 * outQ.z - _2q1 * ax + 4.0f * q2q2 * outQ.z - _2q2 * ay;
            recipNorm = inverse_square(s0 * s0 + s1 * s1 + s2 * s2 + s3 * s3);
            s0 *= recipNorm;
            s1 *= recipNorm;
            s2 *= recipNorm;
            s3 *= recipNorm;
            qDot1 -= GAIN * s0;
            qDot2 -= GAIN * s1;
            qDot3 -= GAIN * s2;
            qDot4 -= GAIN * s3;
        }
        outQ.w += qDot1 * (1.0f / freq);
        outQ.x += qDot2 * (1.0f / freq);
        outQ.y += qDot3 * (1.0f / freq);
        outQ.z += qDot4 * (1.0f / freq);
        recipNorm = inverse_square(outQ.w * outQ.w + outQ.x * outQ.x + outQ.y * outQ.y + outQ.z * outQ.z);
        outQ.w *= recipNorm;
        outQ.x *= recipNorm;
        outQ.y *= recipNorm;
        outQ.z *= recipNorm;
        simplePose.rotation.w = outQ.w;
        simplePose.rotation.x = outQ.x;
        simplePose.rotation.y = outQ.y;
        simplePose.rotation.z = outQ.z;
        return simplePose;
    }
}

using namespace SIMPLE_INPUT;

SimpleInput *SimpleInput::GetInput() {
    static SimpleInput simpleInput;
    return &simpleInput;
}
SimpleInput::SimpleInput() {

}
SimpleInput::~SimpleInput() {

}

SimplePose SimpleInput::GetHeadPose() {
    return sensorInput.headPoseBuffer.GetData();
}

void SimpleInput::Update() {
    RT_LOGI("UpdateInput.");
    sensorInput.sensorData.looper = ALooper_forThread();
    if (nullptr == sensorInput.sensorData.looper) {
        RT_LOGW("looper prepare.");
        sensorInput.sensorData.looper = ALooper_prepare(ALOOPER_PREPARE_ALLOW_NON_CALLBACKS);
    }
    assert(sensorInput.sensorData.looper != nullptr);
    sensorInput.sensorData.sensorManager = ASensorManager_getInstance();
    assert(nullptr != sensorInput.sensorData.sensorManager);
    sensorInput.sensorData.accSensor =
            ASensorManager_getDefaultSensor(sensorInput.sensorData.sensorManager,
                                            ASENSOR_TYPE_ACCELEROMETER);
    assert(nullptr != sensorInput.sensorData.accSensor);
    sensorInput.sensorData.gyroSensor =
            ASensorManager_getDefaultSensor(sensorInput.sensorData.sensorManager,
                                            ASENSOR_TYPE_GYROSCOPE);
    assert(nullptr != sensorInput.sensorData.gyroSensor);
    sensorInput.sensorData.sensorEventQueue =
            ASensorManager_createEventQueue(sensorInput.sensorData.sensorManager,
                                            sensorInput.sensorData.looper, LOOPER_ID_USER,
                                            nullptr, nullptr);
    assert(nullptr != sensorInput.sensorData.sensorEventQueue);
    int32_t ret = -1;
    int32_t minDelay = 0;
    RT_LOGI("Input enable gyro sensor.");
    ret = ASensorEventQueue_enableSensor(sensorInput.sensorData.sensorEventQueue,
                                         sensorInput.sensorData.gyroSensor);
    RT_LOGI("Input enable gyro sensor ret %d",ret);
    assert(ret>=0);
    minDelay = ASensor_getMinDelay(sensorInput.sensorData.gyroSensor);
    ret = ASensorEventQueue_setEventRate(sensorInput.sensorData.sensorEventQueue,
                                         sensorInput.sensorData.gyroSensor, minDelay);
    assert(ret>=0);
    RT_LOGI("Input enable acc sensor.");
    ret = ASensorEventQueue_enableSensor(sensorInput.sensorData.sensorEventQueue,
                                         sensorInput.sensorData.accSensor);
    assert(ret>=0);
    minDelay = ASensor_getMinDelay(sensorInput.sensorData.accSensor);
    ret = ASensorEventQueue_setEventRate(sensorInput.sensorData.sensorEventQueue,
                                         sensorInput.sensorData.accSensor, minDelay);
    assert(ret>=0);
    int32_t loopId,eventNum;
    sensorInput.toWork=true;
    while(sensorInput.toWork){
        void* source = nullptr;
        loopId = ALooper_pollAll(3, nullptr, &eventNum,
                                 reinterpret_cast<void**>(&source));
        RT_LOGI("Poll all: id %d event num %d",loopId,eventNum);
        if ((LOOPER_ID_USER == loopId) && (eventNum>0)) {
            RT_LOGI("LOOPER_ID_USER");
            ASensorEvent sensorEvent;
            while (ASensorEventQueue_getEvents(
                    sensorInput.sensorData.sensorEventQueue, &sensorEvent, 1)>0){
                float delta=0;
                if (sensorEvent.type == ASENSOR_TYPE_ACCELEROMETER) {
                    sensorInput.sensorData.accValid = true;
                    sensorInput.sensorData.accData.x = sensorEvent.acceleration.x;
                    sensorInput.sensorData.accData.y = sensorEvent.acceleration.y;
                    sensorInput.sensorData.accData.z = sensorEvent.acceleration.z;
                }
                else if (sensorEvent.type == ASENSOR_TYPE_GYROSCOPE) {
                    if (sensorInput.sensorData.lastSensorTime != 0
                        && sensorInput.sensorData.accValid) {
                        delta = (sensorEvent.timestamp - sensorInput.sensorData.lastSensorTime) * 1e-9;
                        sensorInput.sensorData.gyroValid = true;
                    }
                    sensorInput.sensorData.gyroData.x =sensorEvent.vector.x ;
                    sensorInput.sensorData.gyroData.y =sensorEvent.vector.y;
                    sensorInput.sensorData.gyroData.z =sensorEvent.vector.z;
                    sensorInput.sensorData.lastSensorTime = sensorEvent.timestamp;
                }
                RT_LOGD("Get time delta %.6f",delta);
                if(delta>=0.001){
                    SimplePose simplePose = get_pose_from_imu(
                            sensorInput.sensorData.gyroData.x,
                            sensorInput.sensorData.gyroData.y,
                            sensorInput.sensorData.gyroData.z,
                            sensorInput.sensorData.accData.x,
                            sensorInput.sensorData.accData.y,
                            sensorInput.sensorData.accData.z);
                    delta=0.0;
                    RT_LOGD("Get simple pose %.6f %.6f %.6f %.6f",simplePose.rotation.x,
                            simplePose.rotation.y,
                            simplePose.rotation.z,
                            simplePose.rotation.w);
                    sensorInput.headPoseBuffer.SetData(simplePose);
                }
            }

        }

    }
    ASensorEventQueue_disableSensor(sensorInput.sensorData.sensorEventQueue,
                                    sensorInput.sensorData.gyroSensor);
    ASensorEventQueue_disableSensor(sensorInput.sensorData.sensorEventQueue,
                                    sensorInput.sensorData.accSensor);
    ASensorManager_destroyEventQueue(sensorInput.sensorData.sensorManager,
                                     sensorInput.sensorData.sensorEventQueue);
    RT_LOGI("Input thread quit.");
}

int32_t SimpleInput::Start() {
    RT_LOGI("Sensor thread start. ");
    std::thread inputThread = std::thread(input_main,this);
    inputThread.detach();
    return 0;
}

void SimpleInput::Stop() {
    sensorInput.toWork = false;
}
GSXR_Result
SimpleInput::GetSupportedDevices(GSXR_Flags64 *deviceTypeFlags){
    return GSXR_Result::GSXR_Error_Feature_Unsupported;
}
GSXR_Result
SimpleInput::GetPrimaryDeviceType(GSXR_Flags64 *deviceTypeFlags){
    return GSXR_Result::GSXR_Error_Feature_Unsupported;
}
GSXR_Result
SimpleInput::SetPrimaryDeviceType(GSXR_DeviceType deviceType){
    return GSXR_Result::GSXR_Error_Feature_Unsupported;
}
GSXR_Result
SimpleInput::GetSupportedDeviceInputType(GSXR_DeviceType deviceType,GSXR_Flags64 *deviceInputTypeFlag){
    return GSXR_Result::GSXR_Error_Feature_Unsupported;
}
GSXR_Result
SimpleInput::IsDeviceConnected(GSXR_DeviceType deviceType,GSXR_Bool32 *connected){
    return GSXR_Result::GSXR_Error_Feature_Unsupported;
}
GSXR_Result
SimpleInput::GetDisplay(GSXR_DeviceType deviceType,GSXR_DisplayId *displayId){
    return GSXR_Result::GSXR_Error_Feature_Unsupported;
}
GSXR_Result
SimpleInput::GetInputIdConfiguration(GSXR_DeviceType deviceType,GSXR_Device_InputType inputType,int inputId,GSXR_Flags64 *configuration){
    return GSXR_Result::GSXR_Error_Feature_Unsupported;
}
GSXR_Result
SimpleInput::GetInputClickStates(GSXR_DeviceType deviceType,GSXR_Device_InputType inputType,GSXR_Flags64 *clicks){
    return GSXR_Result::GSXR_Error_Feature_Unsupported;
}
GSXR_Result
SimpleInput::GetInputTouchStates(GSXR_DeviceType deviceType,GSXR_Device_InputType inputType,GSXR_Flags64 *touchs){
    return GSXR_Result::GSXR_Error_Feature_Unsupported;
}
GSXR_Result
SimpleInput::GetInputAnalogState(GSXR_DeviceType deviceType,GSXR_Device_InputType inputType,int inputId,GSXR_Vector2f *analog){
    RT_LOGD("GetInputAnalogState-deviceType(%d)-inputType(%d)-inputId(%d)");
    if((deviceType == GSXR_DeviceType_GamePad) && (inputType == GSXR_GamePad)){
        Vector2f data = touchDataBuffer.GetData();
        switch (inputId) {
            case GSXR_Gamepad_InputId_Right_Thumbstick_Up:
                (*analog).x = data.x;
                RT_LOGD("Thumbstick_Up(%.3f)",analog->x);
                break;
            case GSXR_Gamepad_InputId_Right_Thumbstick_Dwon:
                (*analog).x = data.x;
                break;
            case GSXR_Gamepad_InputId_Right_Thumbstick_Left:
                (*analog).y = data.y;
                RT_LOGD("Thumbstick_Left(%.3f)",analog->y);
                break;
            case GSXR_Gamepad_InputId_Right_Thumbstick_Right:
                (*analog).y = data.y;
                break;
            default:
                break;
        }
        return GSXR_Result::GSXR_Result_Success;
    }
    return GSXR_Result::GSXR_Result_Success;
}
GSXR_Result
SimpleInput::TriggerVibration(GSXR_DeviceType deviceType,GSXR_Bool32 manual,const GSXR_HapticVibration *haptics){
    return GSXR_Result::GSXR_Error_Feature_Unsupported;
}
GSXR_Result
SimpleInput::GetDegreeOfFreedom(GSXR_DeviceType deviceType,GSXR_DegreeOfFreedom *dof){
    return GSXR_Result::GSXR_Error_Feature_Unsupported;
}
GSXR_Result
SimpleInput::SetSpaceOrigin(GSXR_SpaceOrigin space,const GSXR_Posef *offset){
    return GSXR_Result::GSXR_Error_Feature_Unsupported;
}
GSXR_Result
SimpleInput::ResetOrientationAndPosition(float Yaw){
    return GSXR_Result::GSXR_Error_Feature_Unsupported;
}
GSXR_Result
SimpleInput::GetPoseState(GSXR_DeviceType deviceType,GSXR_Time specifiedTime,GSXR_PoseState *poseState){
    return GSXR_Result::GSXR_Error_Feature_Unsupported;
}
GSXR_Result
SimpleInput::GetSpaceBoundsConfiguration(GSXR_SpaceBoundsConfiguration *boundsConfiguration){
    return GSXR_Result::GSXR_Error_Feature_Unsupported;
}
GSXR_Result
SimpleInput::GetSpaceBoundsRectangle(float *width,float *length){
    return GSXR_Result::GSXR_Error_Feature_Unsupported;
}
GSXR_Result
SimpleInput::GetSpaceBoundsCircle(float *diameter){
    return GSXR_Result::GSXR_Error_Feature_Unsupported;
}
GSXR_Result
SimpleInput::GetSpaceBoundsIrregular(uint32_t pointCount,GSXR_Posef *pointArray){
    return GSXR_Result::GSXR_Error_Feature_Unsupported;
}
GSXR_Result
SimpleInput::SetSpaceBoundsVisible(GSXR_SpaceBoundsVisible visible){
    return GSXR_Result::GSXR_Error_Feature_Unsupported;
}
GSXR_Result
SimpleInput::GetDeviceProperties(GSXR_DeviceType deviceType,GSXR_DeviceProperties *deviceProperties){
    return GSXR_Result::GSXR_Error_Feature_Unsupported;
}
GSXR_Result
SimpleInput::GetBatteryStatus(GSXR_DeviceType deviceType,GSXR_BatteryStatus *batteryStatus){
    return GSXR_Result::GSXR_Error_Feature_Unsupported;
}

void SimpleInput::TouchUpEvent() {
    RT_LOGD("SimpleInput::TouchUpEvent");
    GSXR_Event event={
            .next=nullptr,
            .type=GSXR_EventType ::GSXR_EventType_DeviceInput_BottomToTop_Swiped,
            .timestamp=GetNowTimeNano(),
    };
    SimpleEventManager::GetEventManager()->SubmitEvent(&event);
}

void SimpleInput::TouchDownEvent() {
    RT_LOGD("SimpleInput::TouchDownEvent");
    GSXR_Event event={
            .next=nullptr,
            .type=GSXR_EventType ::GSXR_EventType_DeviceInput_TopToBottom_Swiped,
            .timestamp=GetNowTimeNano(),
    };
    SimpleEventManager::GetEventManager()->SubmitEvent(&event);
}

void SimpleInput::TouchLeftEvent() {
    RT_LOGD("SimpleInput::TouchLeftEvent");
    GSXR_Event event={
            .next=nullptr,
            .type=GSXR_EventType ::GSXR_EventType_DeviceInput_RightToLeft_Swiped,
            .timestamp=GetNowTimeNano(),
    };
    SimpleEventManager::GetEventManager()->SubmitEvent(&event);
}

void SimpleInput::TouchRightEvent() {
    RT_LOGD("SimpleInput::TouchRightEvent");
    GSXR_Event event={
            .next=nullptr,
            .type=GSXR_EventType ::GSXR_EventType_DeviceInput_LeftToRight_Swiped,
            .timestamp=GetNowTimeNano(),
    };
    SimpleEventManager::GetEventManager()->SubmitEvent(&event);
}
void SimpleInput::SingleTapEvent() {
    RT_LOGD("SimpleInput::SingleTapEvent");
}
void SimpleInput::UpdateTouchData(float *inData) {
    float distance = 100.f;
    Vector2f tempData(inData[2]-inData[0],inData[3]-inData[1]);
    tempData.x /= distance;
    tempData.y /= distance;
    tempData.x = tempData.x<-1.f?-1.f:tempData.x;
    tempData.x = tempData.x<1.f?tempData.x:1.f;
    tempData.y = tempData.y<-1.f?-1.f:tempData.y;
    tempData.y = tempData.y<1.f?tempData.y:1.f;
    RT_LOGD("SimpleInput::UpdateTouchData-tempData(%.3f,%.3f)",tempData.x,tempData.y);
    touchDataBuffer.SetData(tempData);
}

void SimpleInput::SingleTouchDownEvent(uint8_t inDown) {
    RT_LOGD("SimpleInput::SingleTouchDownEvent(%d)",inDown);
    if(1==inDown){
        GSXR_Event event={
                .next=nullptr,
                .type=GSXR_EventType ::GSXR_EventType_DeviceInput_Clicked,
                .timestamp=GetNowTimeNano(),
        };
        SimpleEventManager::GetEventManager()->SubmitEvent(&event);
    }else{
        GSXR_Event event={
                .next=nullptr,
                .type=GSXR_EventType ::GSXR_EventType_DeviceInput_Unclicked,
                .timestamp=GetNowTimeNano(),
        };
        SimpleEventManager::GetEventManager()->SubmitEvent(&event);
    }
}