
/**
 *  Created     :   2021-03-1
 *  Copyright (c)  CMCC Technologies, All rights reserved.
 */


#ifndef GSXRRUNTIME_SIMPLEINPUT_H
#define GSXRRUNTIME_SIMPLEINPUT_H

#include "../utils/SimpleMath.h"
#include <android/looper.h>
#include <android/sensor.h>
#include <atomic>
#include "app_api/gsxr.h"

#ifndef LOOPER_ID_USER
#define LOOPER_ID_USER 3
#endif
namespace SIMPLE_INPUT{
/**
 *
 * 定义一些负责提供最基本的设备的位姿信息的数据结构体，用于渲染中的视角变化。
 *
 */

/**
 * 设备sensor数据结构体，陀螺仪，加速度计
 *
 */
    typedef struct SensorData{
        ALooper *             looper;
        ASensorManager*       sensorManager;
        ASensorEventQueue *   sensorEventQueue;
        ASensorRef            gyroSensor;
        ASensorRef            accSensor;
        SIMPLE_MATH::Vector3f gyroData;
        SIMPLE_MATH::Vector3f accData;
        int64_t lastSensorTime;
        bool accValid;
        bool gyroValid;
    }SensorData;
/**
 * 位姿数据结构体
 *
 */
    typedef struct SimplePose{
        SIMPLE_MATH::Vector3f position;
        SIMPLE_MATH::Quaternion rotation;
        SimplePose(): position(),rotation(){}
    }SimplePose;
    /**
     * 定义存贮数据模板类，原子性质。
     */
    template <class T>
    class DataBuffer{
        mutable std::atomic<int32_t> bufferHead;
        mutable std::atomic<int32_t> bufferEnd;
        T data[2];
    public:
        DataBuffer():bufferHead(0),bufferEnd(0){}
        void SetData(T inData){
            const int32_t index = bufferHead.fetch_add(1) & 1;
            data[index ^ 1] = inData;
            bufferEnd.fetch_add(1);
        }
        T GetData() const{
            T outData;
            int32_t head, end, final;
            for(;;) {
                end = bufferEnd.fetch_add(0);
                outData = data[end & 1];
                head = bufferHead.fetch_add(0);
                if (end == head) {
                    return outData;
                }
                outData = data[(head & 1) ^ 1];
                final = bufferHead.fetch_add(0);
                if (final == head) {
                    return outData;
                }
            }
            return outData;
        }
    };
/**
 * 整合设备位姿数据结构体。
 *
 */
    typedef struct SimpleSensorInput{
        SensorData sensorData;
        bool toWork;
        DataBuffer<SimplePose> headPoseBuffer;
    }SimpleSensorInput;

/**
 * 定义 SimpleInput类
 * */
    class SimpleInput{
    public:
        GSXR_Result GetSupportedDevices(GSXR_Flags64 *deviceTypeFlags);
        GSXR_Result GetPrimaryDeviceType(GSXR_Flags64 *deviceTypeFlags);
        GSXR_Result SetPrimaryDeviceType(GSXR_DeviceType deviceType);
        GSXR_Result GetSupportedDeviceInputType(GSXR_DeviceType deviceType,GSXR_Flags64 *deviceInputTypeFlag);
        GSXR_Result IsDeviceConnected(GSXR_DeviceType deviceType,GSXR_Bool32 *connected);
        GSXR_Result GetDisplay(GSXR_DeviceType deviceType,GSXR_DisplayId *displayId);
        GSXR_Result GetInputIdConfiguration(GSXR_DeviceType deviceType,GSXR_Device_InputType inputType,int inputId,GSXR_Flags64 *configuration);
        GSXR_Result GetInputClickStates(GSXR_DeviceType deviceType,GSXR_Device_InputType inputType,GSXR_Flags64 *clicks);
        GSXR_Result GetInputTouchStates(GSXR_DeviceType deviceType,GSXR_Device_InputType inputType,GSXR_Flags64 *touchs);
        GSXR_Result GetInputAnalogState(GSXR_DeviceType deviceType,GSXR_Device_InputType inputType,int inputId,GSXR_Vector2f *analog);
        GSXR_Result TriggerVibration(GSXR_DeviceType deviceType,GSXR_Bool32 manual,const GSXR_HapticVibration *haptics);
        GSXR_Result GetDegreeOfFreedom(GSXR_DeviceType deviceType,GSXR_DegreeOfFreedom *dof);
        GSXR_Result SetSpaceOrigin(GSXR_SpaceOrigin space,const GSXR_Posef *offset);
        GSXR_Result ResetOrientationAndPosition(float Yaw);
        GSXR_Result GetPoseState(GSXR_DeviceType deviceType,GSXR_Time specifiedTime,GSXR_PoseState *poseState);
        GSXR_Result GetSpaceBoundsConfiguration(GSXR_SpaceBoundsConfiguration *boundsConfiguration);
        GSXR_Result GetSpaceBoundsRectangle(float *width,float *length);
        GSXR_Result GetSpaceBoundsCircle(float *diameter);
        GSXR_Result GetSpaceBoundsIrregular(uint32_t pointCount,GSXR_Posef *pointArray);
        GSXR_Result SetSpaceBoundsVisible(GSXR_SpaceBoundsVisible visible);
        GSXR_Result GetDeviceProperties(GSXR_DeviceType deviceType,GSXR_DeviceProperties *deviceProperties);
        GSXR_Result GetBatteryStatus(GSXR_DeviceType deviceType,GSXR_BatteryStatus *batteryStatus);
    public:
        //定义手机触摸屏事件
        void TouchUpEvent();
        void TouchDownEvent();
        void TouchLeftEvent();
        void TouchRightEvent();
        void SingleTapEvent();
        void SingleTouchDownEvent(uint8_t);
        /*
         * @param inData 输入float[4]的浮点数据，（float[0],float[1]）表示touch按下的初始位置，
         *               （float[2],float[3]）表示touch点的更新位置。
         * */
        void UpdateTouchData(float* inData);
    public:
        static SimpleInput* GetInput();
        virtual ~SimpleInput();
        void Update();
        void Stop();
        int32_t Start();
        SimplePose GetHeadPose();
    private:
        SimpleInput();

    private:
        SimpleSensorInput sensorInput;

        //定义手机触摸滑动数据，模拟摇杆
        DataBuffer<SIMPLE_MATH::Vector2f> touchDataBuffer;
    };
}
#endif //GSXRRUNTIME_SIMPLEINPUT_H
