/**
 *  Created     :   2021-03-1
 *  Copyright (c)  CMCC Technologies, All rights reserved.
 */

#include <cassert>
#include "SimpleRender.h"
#include "utils/simple_log.h"
#include "GLES3/gl3.h"
#include <GLES3/gl31.h>
#include <GLES3/gl32.h>
#include "utils/SimpleMath.h"
#include "devices/SimpleInput.h"
#include "utils/simple_utils.h"
#include "utils/SimpleEventManager.h"
using namespace SIMPLE_INPUT;
using namespace SIMPLE_UTILS;
using namespace SIMPLE_EVENT;
namespace SIMPLE_RENDER{
#if DEBUG
#define GL( func ) func; SIMPLE_RENDER::check_gl_error(__LINE__);
#else
#define GL( func ) func;
#endif
    constexpr const char * SIMPLE_VS=
            R"glsl(#version 300 es
    layout (location = 0) in vec3 inPos;
    layout (location = 1) in vec2 inUv;
    out vec2 texCoords;
    void main() {
      gl_Position = vec4(inPos, 1);
      texCoords = inUv;
    })glsl";
    constexpr const char* SIMPLE_FS =
            R"glsl(#version 300 es
    precision mediump float;
    uniform sampler2D texture0;
    in vec2 texCoords;
    out vec4 outColor;
    void main() {
      vec4 color = texture(texture0, texCoords);
      outColor = vec4(color.rgb, 1.0);
      //outColor = vec4(texCoords.x,texCoords.y,0.0,1.0);
    })glsl";
    static void check_gl_error(int line)
    {
        for (GLint error = glGetError(); error; error = glGetError())
        {
            char *pError;
            switch (error)
            {
                case GL_NO_ERROR:                       pError = (char *)"GL_NO_ERROR";                         break;
                case GL_INVALID_ENUM:                   pError = (char *)"GL_INVALID_ENUM";                     break;
                case GL_INVALID_VALUE:                  pError = (char *)"GL_INVALID_VALUE";                    break;
                case GL_INVALID_OPERATION:              pError = (char *)"GL_INVALID_OPERATION";                break;
                case GL_OUT_OF_MEMORY:                  pError = (char *)"GL_OUT_OF_MEMORY";                    break;
                case GL_INVALID_FRAMEBUFFER_OPERATION:  pError = (char *)"GL_INVALID_FRAMEBUFFER_OPERATION";    break;

                default:
                    RT_LOGE("glError (0x%x) %d\n", error, line);
                    return;
            }

            RT_LOGE("glError (%s) %d\n", pError, line);
            return;
        }
        return;
    }
    static GLuint compile_shader(GLenum type,const char * ss){
        GLuint s = GL(glCreateShader(type))
        GL(	glShaderSource( s, 1, &ss, 0 ))
        GL(	glCompileShader( s ))
        GLint result = GL_FALSE;
        glGetShaderiv(s, GL_COMPILE_STATUS, &result);
        if(GL_FALSE==result){
            int32_t length=0;
            GL(glGetShaderiv(s, GL_INFO_LOG_LENGTH, &length))
            if(0==length){
                RT_LOGE("Shader compile failed,but no error!");
                return 0;
            }
            char log[length];
            GL(glGetShaderInfoLog(s, length, nullptr, log))
            RT_LOGE("Shader compile error: %s",log);
            return 0;
        }
        return s;
    }
    static int32_t init_shader(SimpleShader* shader,const char * VS,const char * FS){
        RT_LOGI("init shader");
        shader->verShader = compile_shader(GL_VERTEX_SHADER, SIMPLE_VS);
        assert(shader->verShader);
        shader->fragShader = compile_shader(GL_FRAGMENT_SHADER, SIMPLE_FS);
        assert(shader->fragShader);
        shader->program = GL(glCreateProgram())
        GL(glAttachShader(shader->program,shader->verShader))
        GL(glAttachShader(shader->program,shader->fragShader))
        GL(glLinkProgram(shader->program))
        GLint result = GL_FALSE;
        GL(glGetProgramiv(shader->program, GL_LINK_STATUS, &result))
        if(GL_FALSE == result){
            int32_t length=0;
            GL(glGetProgramiv(shader->program, GL_INFO_LOG_LENGTH, &length))
            if(0==length){
                RT_LOGE("Shader link failed,but no error!");
                return -1;
            }
            char log[length];
            GL(glGetShaderInfoLog(shader->program, length, nullptr, log))
            RT_LOGE("Shader link error: %s",log);
            return -1;
        }else{
            RT_LOGI("Shader linked ok.");
        }
        shader->posLocation = GL(glGetAttribLocation(shader->program,"inPos"))
        RT_LOGI("get shader inPos : %d",shader->posLocation);
        shader->uvLocation = GL(glGetAttribLocation(shader->program,"inUv"))
        RT_LOGI("get shader inUv : %d",shader->uvLocation);
        GL(glDetachShader(shader->program,shader->verShader))
        GL(glDetachShader(shader->program,shader->fragShader))
        GL(glDeleteShader(shader->verShader))
        GL(glDeleteShader(shader->fragShader))
        RT_LOGI("init shader ok");
        return 0;
    }
    static int32_t init_mesh(SimpleEyeRenderData* data, uint32_t segment=1){
        RT_LOGI("init mesh");
        assert(segment);
        SimpleMeshData* meshData = &data->meshData;
        meshData->vertexCount = (segment+1)*(segment+1);
        size_t vertexSize = sizeof (float)*3*meshData->vertexCount;
        size_t uvSize = sizeof (float)*2*meshData->vertexCount;
        meshData->vertices = new float[vertexSize];
        meshData->uvs = new float[uvSize];
        float* pointerVertex = meshData->vertices;
        float* pointerUvs = meshData->uvs;
        for(int32_t iy =0;iy<=segment;iy++){
            for(int32_t ix =0;ix<=segment;ix++){
                ///pos: ll=-1,-1---ru=1,1
                float px = ((float)ix / (float)segment) *2 - 1.0f;
                float py = ((float)iy / (float)segment) *2 - 1.0f;
                float pz = 0.0f;
                *pointerVertex++ = px;
                *pointerVertex++ = py;
                *pointerVertex++ = pz;
                ///st: ll=0,0---ru=1,1
                float s = (float)ix / (float)segment;
                float t = (float)iy / (float)segment;
                *pointerUvs++ = s;
                *pointerUvs++ = t;
            }
        }
        meshData->indexCount = segment*segment*6;
        size_t eleSize = sizeof (int32_t)*meshData->indexCount;
        meshData->index = new int32_t [eleSize];
        int32_t * pointerIndex = meshData->index;
        int32_t ii=0;
        int32_t ia,ib,ic,id;
        for(int32_t iy=0;iy<segment;iy++){
            for(int32_t ix=0;ix<segment;ix++){
                ia = ii;
                ib = ia + 1;
                ic = ib + (segment+1);
                id = ia + (segment+1);
                *pointerIndex++ = ia;
                *pointerIndex++ = ib;
                *pointerIndex++ = ic;
                *pointerIndex++ = ia;
                *pointerIndex++ = ic;
                *pointerIndex++ = id;
                ii++;
            }
            ii++;
        }
        RT_LOGI("init gl buffers.");
        data->eleCount = meshData->indexCount;
        GL(glGenBuffers(1,&data->verticesBuffer))
        GL(glGenBuffers(1,&data->uvBuffer))
        GL(glGenBuffers(1,&data->elementBuffer))
        GL(glBindBuffer(GL_ARRAY_BUFFER,data->verticesBuffer))
        GL(glBufferData(GL_ARRAY_BUFFER,vertexSize,meshData->vertices,GL_STATIC_DRAW))
        GL(glBindBuffer(GL_ARRAY_BUFFER,data->uvBuffer))
        GL(glBufferData(GL_ARRAY_BUFFER,uvSize,meshData->uvs,GL_STATIC_DRAW))
        GL(glBindBuffer(GL_ARRAY_BUFFER,0))
        GL(glBindBuffer(GL_ELEMENT_ARRAY_BUFFER,data->elementBuffer))
        GL(glBufferData(GL_ELEMENT_ARRAY_BUFFER,eleSize,meshData->index,GL_STATIC_DRAW))
        GL(glBindBuffer(GL_ELEMENT_ARRAY_BUFFER,0))
        if(meshData->vertices){
            delete meshData->vertices;
            meshData->vertices= nullptr;
        }
        if(meshData->uvs){
            delete meshData->uvs;
            meshData->uvs= nullptr;
        }
        if(meshData->index){
            delete meshData->index;
            meshData->index= nullptr;
        }
        RT_LOGI("init mesh ok");
        return 0;
    }
    static int32_t render_to_screen(SimpleEyeRender* eyeStereoRender,SimpleRenderLoop* loop){
        RT_LOGI("Render to screen");
        //获取渲染合成层数据
        SimpleCompositorParams& comParam = loop->compositorParams[loop->submitCount%NUM_COMPOSITOR_CHAIN];

        GL(glBindFramebuffer(GL_FRAMEBUFFER,0))
        for(int32_t i=0;i<EYE_COUNT;i++){
            GL(glEnable(GL_SCISSOR_TEST))
            GL(glDisable(GL_CULL_FACE))
            GL(glViewport((0==i)?0:eyeStereoRender->renderEglInfo.surfaceWidth/2,0,
                          eyeStereoRender->renderEglInfo.surfaceWidth/2,
                          eyeStereoRender->renderEglInfo.surfaceHeight))
            GL(glScissor((0==i)?0:eyeStereoRender->renderEglInfo.surfaceWidth/2,0,
                         eyeStereoRender->renderEglInfo.surfaceWidth/2,
                         eyeStereoRender->renderEglInfo.surfaceHeight))
            GL(glClearColor(.2,.2,.2,1.0))
            GL(glClear(GL_COLOR_BUFFER_BIT))
            GL(glUseProgram(eyeStereoRender->eyeRenderData[i].eyeShader.program))
            GL(glBindBuffer(GL_ARRAY_BUFFER,eyeStereoRender->eyeRenderData[i].verticesBuffer))
            GL(glVertexAttribPointer(
                    eyeStereoRender->eyeRenderData[i].eyeShader.posLocation,3,GL_FLOAT,
                    false,0,0))
            GL(glEnableVertexAttribArray(eyeStereoRender->eyeRenderData[i].eyeShader.posLocation))
            GL(glBindBuffer(GL_ARRAY_BUFFER,eyeStereoRender->eyeRenderData[i].uvBuffer))
            GL(glVertexAttribPointer(
                    eyeStereoRender->eyeRenderData[i].eyeShader.uvLocation,2,GL_FLOAT,
                    false,0,0))
            GL(glEnableVertexAttribArray(eyeStereoRender->eyeRenderData[i].eyeShader.uvLocation))
            GL(glActiveTexture(GL_TEXTURE0))

            GL(glBindTexture(GL_TEXTURE_2D, comParam.compositorLayer[i].textureId))
            GL(glBindBuffer(GL_ELEMENT_ARRAY_BUFFER,eyeStereoRender->eyeRenderData[i].elementBuffer))
            GL(glDrawElements(GL_TRIANGLE_STRIP,eyeStereoRender->eyeRenderData[i].eleCount,
                              GL_UNSIGNED_INT,0))
            GL(glUseProgram(0))
            GL(glBindBuffer(GL_ARRAY_BUFFER,0))
            GL(glBindBuffer(GL_ELEMENT_ARRAY_BUFFER,0))
            GL(glDisable(GL_SCISSOR_TEST))
            GL(glEnable(GL_CULL_FACE))
        }
        eglSwapBuffers(eyeStereoRender->renderEglInfo.display,eyeStereoRender->renderEglInfo.renderSurface);
        return 0;
    }
}

using namespace SIMPLE_RENDER;

SimpleRender *SimpleRender::GetRender() {
    static SimpleRender simpleRender;
    return &simpleRender;
}
SimpleRender::SimpleRender():
        foveatedRendering(false),
        multiView(false),
        manual(false){}
SimpleRender::~SimpleRender() {}

GSXR_Result
SimpleRender::CreateRenderer(const GSXR_RendererCreateInfo *createInfo, GSXR_Renderer *renderer) {
    assert(createInfo);
    if(createInfo->graphicsApi!=GSXR_GraphicsApi_OpenGLES){
        RT_LOGE("only support gles now!");
//        return GSXR_Error_Function_Unsupported;
    }

    //检查gl context环境是否初始化
    if (eglGetCurrentContext() == 0 && eglGetCurrentSurface(EGL_DRAW) == 0){
        RT_LOGE("Egl has not initialized!");
        return GSXR_Error_Operation_Failed;
    }

    //获取gles的版本号
    GLint major = 0;
    GLint minor = 0;
    glGetIntegerv(GL_MAJOR_VERSION, &major);
    glGetIntegerv(GL_MINOR_VERSION, &minor);
    eyeStereoRender.renderEglInfo.major = major;
    eyeStereoRender.renderEglInfo.minor = minor;
    RT_LOGI("Gles version major(%d) minor(%d)",major,minor);

    //获取设备所支持的最大纹理尺寸
    int32_t maxTextureSize;
    GL(glGetIntegerv(GL_MAX_TEXTURE_SIZE,&maxTextureSize))
    eyeStereoRender.renderEglInfo.maxTextureSize = maxTextureSize;
    RT_LOGI("Get GL_MAX_TEXTURE_SIZE(%d*%d) ",maxTextureSize,maxTextureSize);

    //获取设备所支持的最大纹理采样数
    int32_t maxSamples;
    GL(glGetIntegerv(GL_MAX_COLOR_TEXTURE_SAMPLES,&maxSamples))
    eyeStereoRender.renderEglInfo.maxSampleCount = maxSamples;
    RT_LOGI("Get GL_MAX_SAMPLES(%d) ",maxSamples);

    //获取设备所支持的最大mipmap等级
    int32_t maxMipLevel;
    GL(glGetIntegerv(GL_TEXTURE_MAX_LEVEL,&maxMipLevel))
    eyeStereoRender.renderEglInfo.maxMipLevel = maxMipLevel;
    RT_LOGI("Get GL_TEXTURE_MAX_LEVEL(%d) ",maxMipLevel);

    //runtime需求的gles的最低版本为3.0
    GSXR_CHECK_ARG_LESS(major,3,"Gles version need equ or greater than 3!",GSXR_Error_Operation_Failed)

    //设置runtime支持的用于合成渲染的纹理格式
    scTextureFormats = {GL_RGBA8,GL_RGBA8_SNORM,GL_SRGB8_ALPHA8};

    //
    eyeStereoRender.renderEglInfo.display = eglGetDisplay(EGL_DEFAULT_DISPLAY);
    eyeStereoRender.renderEglInfo.renderContext = eglGetCurrentContext();
    eyeStereoRender.renderEglInfo.renderSurface = eglGetCurrentSurface(EGL_DRAW);
    eglQuerySurface(eyeStereoRender.renderEglInfo.display,
                    eyeStereoRender.renderEglInfo.renderSurface,
                    EGL_WIDTH, &eyeStereoRender.renderEglInfo.surfaceWidth);
    eglQuerySurface(eyeStereoRender.renderEglInfo.display,
                    eyeStereoRender.renderEglInfo.renderSurface,
                    EGL_HEIGHT, &eyeStereoRender.renderEglInfo.surfaceHeight);

    //初始化合成渲染的shader资源
    SimpleShader eyeShader;
    int32_t ret = init_shader(&eyeShader,SIMPLE_VS,SIMPLE_VS);
    if(ret<0){
        return GSXR_Error_Operation_Failed;
    }

    //初始化合成渲染的几何体资源
    for(int32_t i=0;i<EYE_COUNT;i++){
        eyeStereoRender.eyeRenderData[i].eyeShader = eyeShader;
        ret = init_mesh(&eyeStereoRender.eyeRenderData[i]);
        if(ret<0){
            RT_LOGE("init mesh error!");
            break;
        }
    }
    if(ret<0){
        return GSXR_Error_Operation_Failed;
    }

    //返回创建好的GSXR_Renderer对象
    *renderer = (GSXR_Renderer)(&eyeStereoRender);

    //发送一个设备已连接状态的事件。
    GSXR_Event event={
            .next=nullptr,
            .type=GSXR_EventType ::GSXR_EventType_DeviceState_Connected,
            .timestamp=GetNowTimeNano(),
            .gsxrEventData={
                    .gsxr_devicesatate_eventdata={
                            .deviceType=GSXR_DeviceType::GSXR_DeviceType_HMD_VR,
                    },
            },
    };
    SimpleEventManager::GetEventManager()->SubmitEvent(&event);

    RT_LOGI("create render ok.");
    return GSXR_Result_Success;
}
GSXR_Result
SimpleRender::DestroyRenderer(GSXR_Renderer renderer){
    assert(renderer==&eyeStereoRender);
    if(eyeStereoRender.eyeRenderData->eyeShader.program){
        GL(glDeleteProgram(eyeStereoRender.eyeRenderData->eyeShader.program))
    }
    for(int32_t i=0;i<EYE_COUNT;i++){
        eyeStereoRender.eyeRenderData[i].Reset();
    }
    return GSXR_Result_Success;
}
GSXR_Result
SimpleRender::GetViewSetCount(uint32_t* count){
    //runtime目前设计仅支持一种视图类型GSXR_ViewType_Primary_Stereo
    //则视图集个数设置为1
    *count = VIEW_SET_COUNT;
    return GSXR_Result_Success;
}
GSXR_Result
SimpleRender::GetViewSetConfigurations(uint32_t count,GSXR_ViewSetConfiguration* configurations){
    RT_LOGI("GetViewSetConfigurations");

    //检查count不可为0
    GSXR_CHECK_ARG_EQU(count,0,"Arg: count can not set to 0!",GSXR_Error_Argument_Invalid);

    //设置视图集参数
    GSXR_ViewSetConfiguration viewSetConfiguration={
            .next = nullptr,
            .viewType = GSXR_ViewType_Primary_Stereo,//视图集的视图类型
            .viewCount = 2,//视图view的个数，2为左右双眼视图
            .textureFormatCount = (uint32_t)scTextureFormats.size(),//合成渲染支持的纹理格式
            .targetWidth = EYE_BUFFER_WIDTH,//设置建议的目标纹理的宽度
            .maxWidth = (uint32_t)eyeStereoRender.renderEglInfo.maxTextureSize,//设置设备所支持的最大目标纹理宽度
            .targetHeight = EYE_BUFFER_HEIGHT,//设置建议的目标纹理的高度
            .maxHeight = (uint32_t)eyeStereoRender.renderEglInfo.maxTextureSize,//设置设备所支持的最大目标纹理高度
            .targetSampleCount=2,//设置纹理采样数，需大于0
            .maxSampleCount=(uint32_t)eyeStereoRender.renderEglInfo.maxSampleCount,//设置最大纹理采样数
            .fps=60,
            .maxLayerCount=LAYER_COUNT,
            .ipd=IPD_INT,
            .fovHRad=FOV_H*DEG2RAD,
            .fovVRad=FOV_V*DEG2RAD,
    };
    configurations[0] = viewSetConfiguration;

    RT_LOGI("get view configs ok.");
    return GSXR_Result_Success;
}
GSXR_Result
SimpleRender::GetTextureQueueOptions(GSXR_Renderer renderer,GSXR_Flags64* recommendedOptions,GSXR_Flags64* supportedOptions){
    RT_LOGI("GetTextureQueueOptions");
    assert(renderer == &eyeStereoRender);
    *recommendedOptions = GSXR_TextureQueueOptions_Manual_Management;
    *supportedOptions = GSXR_TextureQueueOptions_Manual_Management
            |GSXR_TextureQueueOptions_Color_Attachment;
    RT_LOGI("Get texture queue options ok.");
    return GSXR_Result_Success;
}
GSXR_Result
SimpleRender::GetSupportedTextureFormats(GSXR_Renderer renderer,uint32_t formatCount,GSXR_TextureFormat* formats){
    RT_LOGI("GetSupportedTextureFormats(%d)",formatCount);
    assert(renderer == &eyeStereoRender);
    assert(formatCount == scTextureFormats.size());
    memcpy(formats,scTextureFormats.data(),sizeof(GSXR_TextureFormat)*formatCount);
    return GSXR_Result_Success;
}
GSXR_Result
SimpleRender::CreateTextureQueue(GSXR_Renderer renderer,const GSXR_TextureQueueCreateInfo *createInfo,GSXR_TextureQueue *textureQueue){
    RT_LOGD("Create swap chain");
    //验证创建的渲染器是否正确
    GSXR_CHECK_ARG_NO_EQU(renderer,&eyeStereoRender,"GSXR_Renderer not match!",GSXR_Error_Argument_Invalid)

    //如果是选择了GSXR应用自己创建纹理队列，则返回。
    if(createInfo->options & GSXR_TextureQueueOptions_Manual_Management){
        textureQueue = nullptr;
        manual = true;
        RT_LOGD("App manual create swap chain");
        return GSXR_Result_Success;
    }
    RT_LOGI("Runtime create swap chain");
    //验证纹理小大，采样数，纹理数组大小，均不可为0
    GSXR_CHECK_ARG_EQU(0,createInfo->width,"Texture width cannot set to 0!",GSXR_Error_Argument_Invalid)
    GSXR_CHECK_ARG_EQU(0,createInfo->height,"Texture height cannot set to 0!",GSXR_Error_Argument_Invalid)
    GSXR_CHECK_ARG_EQU(0,createInfo->sampleCount,"Texture sampleCount cannot set to 0!",GSXR_Error_Argument_Invalid)

    //arrayLayerCount
    // 值为1时表示纹理目标类型为GL_TEXTURE_2D
    // 值为2时表示纹理目标类型为GL_TEXTURE_2D_ARRAY
    GSXR_CHECK_ARG_EQU(0,createInfo->arrayLayerCount,"Texture arrayLayerCount cannot set to 0!",GSXR_Error_Argument_Invalid)

    //目前只支持平面投影即单张图像，暂时不支持6张图像的cube立体全景投影
    GSXR_CHECK_ARG_NO_EQU(1,createInfo->faceCount,"FaceCount need set 1!",GSXR_Error_Argument_Invalid)

    //验证纹理格式是否支持
    bool formatSupported = false;
    for(int64_t scTextureFormat : scTextureFormats){
        if(createInfo->format == scTextureFormat){
            formatSupported = true;
            break;
        }
    }
    GSXR_CHECK_ARG_FALSE(formatSupported,"Texture format not support!",GSXR_Error_Argument_Invalid)

    //找到一个可用索引
    int32_t swapChainIndex = -1;
    for(int32_t i=0;i<MAX_SWAP_CHAIN_NUM;i++){
        if(swapChains[i]== nullptr){
            swapChainIndex = i;
            break;
        }
    }
    if(swapChainIndex==-1){
        RT_LOGE("Swap chain is full.");
        return GSXR_Error_Operation_Failed;
    }else{
        RT_LOGD("Find a swap chain index(%d)",swapChainIndex);
    }
    //创建交换链对象
    swapChains[swapChainIndex] = new SimpleSwapChain();
    SimpleSwapChain& swapChain = *swapChains[swapChainIndex];
    swapChain.scIndex=0;
    GLuint textureTarget=GL_TEXTURE_2D;
//    int32_t textureCount=0;
    if(createInfo->arrayLayerCount>1){
        swapChain.textureTargetType = Texture_array;
        textureTarget = GL_TEXTURE_2D_ARRAY;
//        textureCount=1;
        multiView=true;

    }else{
        swapChain.textureTargetType = Texture_normal;
        textureTarget = GL_TEXTURE_2D;
//        textureCount=2;
        multiView=false;
    }
    RT_LOGD("multiView(%d)",multiView);
    swapChain.textureTargetFormat = createInfo->format;
    swapChain.eyeTargetSize.targetEyeWidth = createInfo->width;
    swapChain.eyeTargetSize.targetEyeHeight = createInfo->height;
    swapChain.mipLevel = createInfo->mipLevelCount;
    RT_LOGD("mipLevel(%d)",createInfo->mipLevelCount);
    GLint filterParam = swapChain.mipLevel>1?GL_LINEAR_MIPMAP_LINEAR:GL_LINEAR;
    swapChain.chainLength = SWAP_CHAIN_LENGTH;
    for(int32_t i =0;i<SWAP_CHAIN_LENGTH;i++){
//        auto texture = reinterpret_cast<GLuint *>(&swapChain.textureElement[i].texture);
        GL(glGenTextures(1, &swapChain.textureElement[i]))
        GL(glBindTexture(textureTarget, swapChain.textureElement[i]))
        GL(glTexParameteri(textureTarget, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE))
        GL(glTexParameteri(textureTarget, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE))
        GL(glTexParameteri(textureTarget, GL_TEXTURE_MIN_FILTER, filterParam))
        GL(glTexParameteri(textureTarget, GL_TEXTURE_MAG_FILTER, GL_LINEAR))
        if(multiView){
            GL(glTexStorage3D(textureTarget, swapChain.mipLevel,
                              swapChain.textureTargetFormat,
                              swapChain.eyeTargetSize.targetEyeWidth,
                              swapChain.eyeTargetSize.targetEyeHeight,2))
        }else{
            GL(glTexStorage2D(textureTarget, swapChain.mipLevel,
                              swapChain.textureTargetFormat,
                              swapChain.eyeTargetSize.targetEyeWidth,
                              swapChain.eyeTargetSize.targetEyeHeight))
        }
        GL(glBindTexture(GL_TEXTURE_2D, 0))
        RT_LOGD("glGenTextures texture(%d)",swapChain.textureElement[i]);
    }
//    swapChains.emplace_back(swapChain);
//    auto key = (GSXR_TextureQueue)&(swapChains.back());
    auto key = reinterpret_cast<GSXR_TextureQueue>(swapChains[swapChainIndex]);
    swapChainsMap[key]=swapChainIndex;
    *textureQueue = key;
    RT_LOGD("textureQueue(%p)",*textureQueue);
//    *textureQueue = GSXR_NULL_HANDLE;
//    renderLoop.swapChain.textureTargetFormat = GL_TEXTURE_2D;
//    for(int32_t i=0;i<EYE_COUNT;i++){
//        renderLoop.swapChain.eyeTargetSize[i].targetEyeWidth = EYE_BUFFER_WIDTH;
//        renderLoop.swapChain.eyeTargetSize[i].targetEyeHeight = EYE_BUFFER_HEIGHT;
//        renderLoop.swapChain.eyeFrustum[i].left = DEG2RAD*(FOV_H/2);
//        renderLoop.swapChain.eyeFrustum[i].right = DEG2RAD*(FOV_H/2);
//        renderLoop.swapChain.eyeFrustum[i].up = DEG2RAD*(FOV_V/2);
//        renderLoop.swapChain.eyeFrustum[i].down = DEG2RAD*(FOV_V/2);
//    }
    RT_LOGI("Create texture queue ok.");
    return GSXR_Result_Success;
}
GSXR_Result
SimpleRender::DestroyTextureQueue(GSXR_TextureQueue textureQueue){
    RT_LOGD("Destroy swap chain");
    auto sc = swapChainsMap.find(textureQueue);
    if(sc == swapChainsMap.end()){
        RT_LOGE("Invalid textureQueue.");
        return GSXR_Error_Operation_Failed;
    }

    delete swapChains[sc->second];
    swapChains[sc->second]= nullptr;
    swapChainsMap.erase(sc);

    return GSXR_Result_Success;
}
GSXR_Result
SimpleRender::GetTextureCount(GSXR_TextureQueue textureQueue,uint32_t *count){
    return GSXR_Error_Operation_Failed;
}
GSXR_Result
SimpleRender::WaitAndAcquireAvailableTexture(GSXR_TextureQueue textureQueue,GSXR_Duration timeout,GSXR_TextureElement *textureElement){
    GSXR_CHECK_ARG_NULL(textureQueue,"TextureQueue is null!",GSXR_Error_Handle_Invalid)

    for(int32_t i=0;i<MAX_SWAP_CHAIN_NUM;i++){
        if(swapChains[i]!= nullptr){
            SimpleSwapChain* swapChain = swapChains[i];
            RT_LOGD("asc(%p): index(%d)",swapChain,swapChain->scIndex);
            for(auto ti : swapChain->textureElement){
                RT_LOGD("asc:  texture(%d)",ti);
            }
        }
    }

    auto iterator = swapChainsMap.find(textureQueue);
    if(iterator == swapChainsMap.end()){
        RT_LOGE("TextureQueue is not match!");
        return GSXR_Error_Operation_Failed;
    }
    RT_LOGD("iterator->second(%d)",iterator->second);
    SimpleSwapChain* swapChain = swapChains[iterator->second];
    RT_LOGD("TextureQueue index(%d)->swapChain(%p)!",swapChain->scIndex,swapChain);
    for(auto ti : swapChain->textureElement){
        RT_LOGD("swapChain(%p):  texture(%d)",swapChain,ti);
    }
    swapChain->scIndex = (swapChain->scIndex+1)%SWAP_CHAIN_LENGTH;
    RT_LOGE("TextureQueue index(%d)!",swapChain->scIndex);
    GSXR_TextureElement element={
            .next=nullptr,
            .index = swapChain->scIndex,
            .texture = GSXR_Texture(&swapChain->textureElement[swapChain->scIndex]),
    };
    *textureElement = element;
//    RT_LOGD("textureElement swapChain(%p) scIndex(%u) index(%u)",
//            swapChain,swapChain->scIndex,swapChain->textureElement[swapChain->scIndex].index);
    RT_LOGD("textureElement swapChain(%p) scIndex(%u) texture(%d)",
            swapChain,swapChain->scIndex,swapChain->textureElement[swapChain->scIndex]);
    return GSXR_Result_Success;
}
GSXR_Result
SimpleRender::ReleaseTexture(GSXR_TextureQueue textureQueue,const GSXR_TextureElement *textureElement){
    return GSXR_Result_Success;
}
GSXR_Result
SimpleRender::StartRenderLooper(GSXR_Renderer renderer,const GSXR_RenderLooperStartInfo *startInfo,GSXR_RenderLooper *looper){
    assert(renderer == &eyeStereoRender);

    renderLoop.submitCount = 0;
    renderLoop.loopState = SimpleRenderLoop::RLS_Starting;
    *looper=&renderLoop;

    RT_LOGI("start render loop ok.");
    return GSXR_Result_Success;
}
GSXR_Result
SimpleRender::StopRenderLooper(GSXR_RenderLooper looper){
    RT_LOGI("Stop render loop.");
    renderLoop.loopState = SimpleRenderLoop::RLS_Unavailable;
    for(auto & sc : swapChains){
        delete sc;
        sc = nullptr;
    }
    swapChainsMap.clear();
    return GSXR_Result_Success;
}
GSXR_Result
SimpleRender::SetRenderLooperThread(GSXR_RenderLooper looper,uint32_t threadId){
    return GSXR_Result_Success;
}
GSXR_Result
SimpleRender::WaitFrame(GSXR_RenderLooper looper,GSXR_Time *predictedDisplayTime){
    assert(looper == &renderLoop);
    *predictedDisplayTime = 0;
    return GSXR_Result_Success;
}
GSXR_Result
SimpleRender::GetViewPoseState(GSXR_RenderLooper looper,GSXR_Time specifiedTime,uint32_t viewCount,GSXR_ViewPoseState *viewPoseState){
    assert(looper == &renderLoop);
    assert(viewCount);
    if(viewCount<=EYE_COUNT){
        for(int32_t i=0;i<viewCount;i++){
            viewPoseState[i].fov={
                    .angleLeft = DEG2RAD*(FOV_H/2),
                    .angleRight = DEG2RAD*(FOV_H/2),
                    .angleUp = DEG2RAD*(FOV_V/2),
                    .angleDown = DEG2RAD*(FOV_V/2),
            };
            RT_LOGI("viewPoseState.fov: (u-%.3f,d-%.3f,l-%.3f,r-%.3f)",
                  viewPoseState[i].fov.angleUp,
                  viewPoseState[i].fov.angleDown,
                  viewPoseState[i].fov.angleLeft,
                  viewPoseState[i].fov.angleRight);
            SimplePose simplePose = SimpleInput::GetInput()->GetHeadPose();
            viewPoseState[i].pose.position.x = 0;
            viewPoseState[i].pose.position.y = 1.5;
            viewPoseState[i].pose.position.z = 0;
            viewPoseState[i].pose.orientation.x = simplePose.rotation.x;
            viewPoseState[i].pose.orientation.y = simplePose.rotation.y;
            viewPoseState[i].pose.orientation.z = simplePose.rotation.z;
            viewPoseState[i].pose.orientation.w = simplePose.rotation.w;
            RT_LOGI("viewPoseState.orientation: (x-%.3f,y-%.3f,z-%.3f,w-%.3f)",
                    viewPoseState[i].pose.orientation.x,
                    viewPoseState[i].pose.orientation.y,
                    viewPoseState[i].pose.orientation.z,
                    viewPoseState[i].pose.orientation.w);
        }
    }
    return GSXR_Result_Success;
}
GSXR_Result
SimpleRender::SubmitFrame(GSXR_RenderLooper looper,const GSXR_FrameSubmitInfo *submitInfo){
    //验证loop是否匹配
    GSXR_CHECK_ARG_NO_EQU(looper,&renderLoop,"GSXR_RenderLooper not match!",GSXR_Error_Handle_Invalid)
    //验证submitInfo是否为空指针
    GSXR_CHECK_ARG_NULL(submitInfo,"SubmitInfo is null!",GSXR_Error_Handle_Invalid)
    //验证提交的帧信息中layer数量是否为0
    GSXR_CHECK_ARG_EQU(submitInfo->layerCount,0,"SubmitInfo layerCount cannot set to 0!",GSXR_Error_Handle_Invalid)
    //验证layer数据指针是否为空
    GSXR_CHECK_ARG_NULL(submitInfo->layers,"SubmitInfo is null!",GSXR_Error_Handle_Invalid)

    RT_LOGI("submit time %ld ",submitInfo->displayTime);
    RT_LOGI("submit layer count %d ",submitInfo->layerCount);

    //验证纹理队列是否为GSXR应用创建
//    bool manual = false;
//    if(swapChains.empty()){
//        manual=true;
//    }

    //提交渲染帧次数增加1
    uint32_t nextCount = renderLoop.submitCount + 1;
    SimpleCompositorParams& comParam = renderLoop.compositorParams[nextCount%NUM_COMPOSITOR_CHAIN];
    uint32_t comLayerIndex = 0;

    //获取提交的layer信息
    for(int32_t i=0; i<submitInfo->layerCount;i++){
        RT_LOGI("submit layer%d type %d ",i,submitInfo->layers[i]->layerType);
        GSXR_CompositionLayerType type = submitInfo->layers[i]->layerType;
        switch (type) {
            //目前只支持透视投影类型视图
            case GSXR_CompositionLayerType_Projection:
            {
                auto layer = (GSXR_CompositionLayerProjection*)submitInfo->layers[i];
                RT_LOGI("Project viewCount %d",layer->viewCount);
                //目前只支持左右双视图类型
                if(2==layer->viewCount){
                    //获取提交的纹理信息到渲染合成层数据
                    //先验证纹理是否为GSXR应用所创建
                    if(!manual){
                        //runtime的纹理队列有数据
                        //views[0]与views[1]的纹理队列应为同一个队列，验证纹理队列是否为空
                        if((nullptr == layer->views[0].texSubImage.textureQueue)){
                            //应用提交的渲染队列为空，当作应用自行创建的纹理队列处理
                            manual = true ;
                        }
                    }
                    SimpleTextureType textureType = Texture_normal;
                    if(manual){
                        //验证纹理类型
                        bool isTextureArray = false;
                        int32_t count =0;
                        do{
                            if(layer->views[count].texSubImage.arrayLayerIndex>0){
                                isTextureArray = true;
                            }
                            count++;
                        }while(count<2);
                        if(isTextureArray){
                            textureType = Texture_array;
                        }
                    }else{
                        //验证纹理队列
                        auto sc = swapChainsMap.find(layer->views[0].texSubImage.textureQueue);
                        if(sc == swapChainsMap.end()){
                            //未找到对应的纹理队列，返回操作失败！
                            RT_LOGE("Can not get texture queue!");
                            return GSXR_Error_Operation_Failed;
                        }
                        SimpleSwapChain* swapChain = swapChains[sc->second];
                        textureType = swapChain->textureTargetType;
                    }

                    comParam.compositorLayer[comLayerIndex].textureId =
                            *(int32_t*)(layer->views[0].texSubImage.textureElement.texture);
                    comParam.compositorLayer[comLayerIndex].textureType = textureType;
                    comParam.compositorLayer[comLayerIndex].eyeMask = layer->views[0].eyeMask;
                    comLayerIndex++;
                    comParam.compositorLayer[comLayerIndex].textureId =
                            *(int32_t*)(layer->views[1].texSubImage.textureElement.texture);
                    comParam.compositorLayer[comLayerIndex].textureType = textureType;
                    comParam.compositorLayer[comLayerIndex].eyeMask = layer->views[1].eyeMask;
                    comLayerIndex++;
//                    RT_LOGI("Right eye texture %d texture(%d)",renderLoop.swapChain.texture[1][renderLoop.frameIndex],
//                            *(uint32_t*)(layer->views[1].texSubImage.textureElement.texture));
                }
            }
                break;
            case GSXR_CompositionLayerType_Quad:
                break;
            default:
                RT_LOGE("Submitted layer%d the type %d not support! ",i,
                        submitInfo->layers[i]->layerType);
                break;
        }
    }
    renderLoop.submitCount = nextCount;

    render_to_screen(&eyeStereoRender,&renderLoop);

    return GSXR_Result_Success;
}
GSXR_Result
SimpleRender::GetRenderFunctions(GSXR_RenderLooper looper,GSXR_Flags64 *recommendedFunctions,GSXR_Flags64 *supportedFunctions){
    assert(looper == &renderLoop);
    if(!foveatedRendering){
        *recommendedFunctions = 0;
        *supportedFunctions = 0;
    }
    RT_LOGI("Get render functions ok.");
    return GSXR_Result_Success;
}
GSXR_Result
SimpleRender::PreRenderFrame(GSXR_RenderLooper looper,const GSXR_FrameRenderInfo *preRenderInfo){
    return GSXR_Result_Success;
}
GSXR_Result
SimpleRender::PostRenderFrame(GSXR_RenderLooper looper,const GSXR_FrameRenderInfo *postRenderInfo){
    return GSXR_Result_Success;
}
GSXR_Result
SimpleRender::GetFoveatedParameters(GSXR_RenderLooper looper,GSXR_Bool32 *enabled,uint32_t viewCount,GSXR_FoveatedParameters *foveatedParams){
    assert(looper == &renderLoop);
    if(!foveatedRendering){
        return GSXR_Error_Feature_Unavailable;
    }
    return GSXR_Result_Success;
}


