
#ifndef GSXRRUNTIME_SIMPLERENDER_H
#define GSXRRUNTIME_SIMPLERENDER_H
/**
 *  Created     :   2021-03-1
 *  Copyright (c)  CMCC Technologies, All rights reserved.
 */

#include <cstdint>
#include <EGL/egl.h>
#include "app_api/gsxr.h"
#include <map>
#include <vector>

#define EYE_COUNT 2
#define SWAP_CHAIN_LENGTH 3

#define FOV_H 90.0
#define FOV_V 90.0
#define EYE_BUFFER_WIDTH 1024
#define EYE_BUFFER_HEIGHT 1024
#define IPD_INT 64

#define LAYER_COUNT 16

#define NUM_COMPOSITOR_CHAIN 4

#define MAX_SWAP_CHAIN_NUM 16

#define VIEW_SET_COUNT 1


namespace SIMPLE_RENDER {
/**
 * 定义一些最基本的Opengl-es的渲染管线的运行，包括mesh,shader等资源的创建等。
 * 上屏渲染即指把图像渲染到屏幕上，是Runtime主要的实现内容，涉及到XR行业内的许多技术性指标，例如：
 * 画面延迟，画面反畸变，色差矫正等等。目前代码实现了最基本的同步渲染上屏机制，把GSXR应用绘制的左眼视角
 * 场景和右眼视角场景内容分别同步渲染到屏幕的左半侧屏幕和右半侧屏幕上。
 *
 */

    /**
     * EGL信息数据结构体
     */
    typedef struct SimpleEglInfo{
        int32_t major = 0;
        int32_t minor = 0;
        EGLDisplay display;
        EGLContext renderContext;
        EGLSurface renderSurface;
        int32_t surfaceWidth;
        int32_t surfaceHeight;
        int32_t maxTextureSize;
        int32_t maxSampleCount;
        int32_t maxMipLevel;
    }SimpleEglInfo;
    /**
     * 多边形数据结构体，记录点位置，uv,索引等信息。
     */
    typedef struct SimpleMeshData {
        float* vertices;///x,y,z
        float* uvs;///s,t
        int32_t vertexCount;
        int32_t *index;
        int32_t indexCount;
        SimpleMeshData():vertices(nullptr),uvs(nullptr),vertexCount(0),index(nullptr),indexCount(0){}
        void Reset(){
            vertices= nullptr;
            uvs= nullptr;
            index= nullptr;
            vertexCount=0;
            indexCount=0;
        }
    } SimpleMeshData;
    /**
     * Shader数据结构体
     */
    typedef struct SimpleShader {
        uint32_t program;
        uint32_t verShader;
        uint32_t fragShader;
        int32_t posLocation;
        int32_t uvLocation;
        SimpleShader():program(0),verShader(0),fragShader(0),posLocation(-1),uvLocation(-1){}
        void Reset(){
            program=0;
            verShader=0;
            fragShader=0;
            posLocation=-1;
            uvLocation=-1;
        }
    } SimpleShader;
    /**
     * 渲染数据整合数据结构体
     */
    typedef struct SimpleEyeRenderData{///for one eye rendering.
        uint32_t verticesBuffer;
        uint32_t uvBuffer;
        uint32_t elementBuffer;
        int32_t eleCount;
        SimpleShader eyeShader;
        SimpleMeshData meshData;
        SimpleEyeRenderData():verticesBuffer(0),uvBuffer(0),elementBuffer(0),eleCount(0),eyeShader(),meshData(){}
        void Reset(){
            verticesBuffer=0;
            uvBuffer=0;
            elementBuffer=0;
            eleCount=0;
            eyeShader.Reset();
            meshData.Reset();
        }
    }SimpleEyeRenderData;
    /**
    * **SimpleEyeRender**结构体定义了用于执行Opengl-es渲染管线运行所需要的最基本的渲染资源，例如Shader,
    * 多边形点位置坐标，UV信息坐标等等，代码中提供了相应的**SimpleMeshData**,**SimpleShader**,
    * **SimpleEyeRenderData**等数据结构体
    */
    typedef struct SimpleEyeRender{
        SimpleEyeRenderData eyeRenderData[EYE_COUNT];
        SimpleEglInfo renderEglInfo;
    }SimpleEyeRender;
    /**
     * 目标体尺寸数据结构体
     */
    typedef struct SimpleEyeTargetSize{
        uint32_t targetEyeWidth;
        uint32_t targetEyeHeight;
    }SimpleEyeTargetSize;
    /**
     * 视锥体数据结构体
     */
    typedef struct SimpleEyeFrustum{
        float left;
        float right;
        float up;
        float down;
    }SimpleEyeFrustum;

    /**
     *
     * */
     enum SimpleTextureType{
         Texture_normal,
         Texture_array,
     };
    typedef struct {
        int32_t textureId;
        SimpleTextureType textureType;
        GSXR_Eye eyeMask;
    }SimpleCompositorLayer;
     typedef struct {
         SimpleCompositorLayer compositorLayer[LAYER_COUNT];
     }SimpleCompositorParams;
    /**
* **SimpleSwapChain**交换链数据结构体,为了防止纹理资源的访问冲突，Runtime实现端或者GXSR应用渲染
* 端会创建一个纹理队列，在每帧渲染时，循环使用该纹理队列，GSXR应用会通过调用GSXR API接口
* **GSXR_GetTextureQueueOptions**时查询Runtime提供的创建纹理队列的建议及支持情况。
*
*/
    typedef struct SimpleSwapChain{
        int64_t textureTargetFormat;
        uint32_t mipLevel;
        uint32_t chainLength;
        SimpleTextureType textureTargetType;
        SimpleEyeTargetSize eyeTargetSize;
        SimpleEyeFrustum eyeFrustum;
        uint32_t textureElement[SWAP_CHAIN_LENGTH];
        uint32_t scIndex;
    }SimpleSwapChain;
    /**
* **SimpleRenderLoop**
* 用于渲染器渲染的状态，帧信息等内容的管理任务
* */
    typedef struct SimpleRenderLoop{
        enum RenderLoopState{
            RLS_Unavailable = -1,
            RLS_Available = 0,
            RLS_Starting = 1,
            RLS_Ready = 2,
            RLS_Focused = 3,
            RLS_Unfocused = 4,
            RLS_Unready = 5,
            RLS_Stopping = 6,
        };
        uint32_t submitCount;
        RenderLoopState loopState;
        SimpleCompositorParams compositorParams[NUM_COMPOSITOR_CHAIN];
        SimpleRenderLoop():submitCount(0),loopState(RLS_Unavailable){}
    }SimpleRenderLoop;
    /**
     * 定义SimpleRender类
     * */
    class SimpleRender{
    public:
        static SimpleRender* GetRender();
        virtual~SimpleRender();

    public:
        GSXR_Result CreateRenderer(const GSXR_RendererCreateInfo* createInfo,GSXR_Renderer* renderer);
        GSXR_Result DestroyRenderer(GSXR_Renderer renderer);
        GSXR_Result GetViewSetCount(uint32_t* count);
        GSXR_Result GetViewSetConfigurations(uint32_t count,GSXR_ViewSetConfiguration* configurations);
        GSXR_Result GetTextureQueueOptions(GSXR_Renderer renderer,GSXR_Flags64* recommendedOptions,GSXR_Flags64* supportedOptions);
        GSXR_Result GetSupportedTextureFormats(GSXR_Renderer renderer,uint32_t formatCount,GSXR_TextureFormat* formats);
        GSXR_Result CreateTextureQueue(GSXR_Renderer renderer,const GSXR_TextureQueueCreateInfo *createInfo,GSXR_TextureQueue *textureQueue);
        GSXR_Result DestroyTextureQueue(GSXR_TextureQueue textureQueue);
        GSXR_Result GetTextureCount(GSXR_TextureQueue textureQueue,uint32_t *count);
        GSXR_Result WaitAndAcquireAvailableTexture(GSXR_TextureQueue textureQueue,GSXR_Duration timeout,GSXR_TextureElement *textureElement);
        GSXR_Result ReleaseTexture(GSXR_TextureQueue textureQueue,const GSXR_TextureElement *textureElement);
        GSXR_Result StartRenderLooper(GSXR_Renderer renderer,const GSXR_RenderLooperStartInfo *startInfo,GSXR_RenderLooper *looper);
        GSXR_Result StopRenderLooper(GSXR_RenderLooper looper);
        GSXR_Result SetRenderLooperThread(GSXR_RenderLooper looper,uint32_t threadId);
        GSXR_Result WaitFrame(GSXR_RenderLooper looper,GSXR_Time *predictedDisplayTime);
        GSXR_Result GetViewPoseState(GSXR_RenderLooper looper,GSXR_Time specifiedTime,uint32_t viewCount,GSXR_ViewPoseState *viewPoseState);
        GSXR_Result SubmitFrame(GSXR_RenderLooper looper,const GSXR_FrameSubmitInfo *submitInfo);
        GSXR_Result GetRenderFunctions(GSXR_RenderLooper looper,GSXR_Flags64 *recommendedFunctions,GSXR_Flags64 *supportedFunctions);
        GSXR_Result PreRenderFrame(GSXR_RenderLooper looper,const GSXR_FrameRenderInfo *preRenderInfo);
        GSXR_Result PostRenderFrame(GSXR_RenderLooper looper,const GSXR_FrameRenderInfo *postRenderInfo);
        GSXR_Result GetFoveatedParameters(GSXR_RenderLooper looper,GSXR_Bool32 *enabled,uint32_t viewCount,GSXR_FoveatedParameters *foveatedParams);

    private:
        SimpleRender();

    private:

        //是否启用了注视点渲染项
        bool foveatedRendering;

        //是否启用了multi-view渲染，
        // runtime会应对该项设置目标纹理的类型为GL_TEXTURE_2D或者GL_TEXTURE_2D_ARRAY
        bool multiView;

//        //用于swap chain的计数
//        int32_t swapChainIndex;
//        std::vector<SimpleSwapChain> swapChains;
        SimpleSwapChain* swapChains[MAX_SWAP_CHAIN_NUM];

        SimpleRenderLoop renderLoop;
        SimpleEyeRender eyeStereoRender;
        std::vector<GSXR_TextureFormat> scTextureFormats;
        std::map<GSXR_TextureQueue,int32_t> swapChainsMap;
        bool manual;
    };
}//namespace
#endif //GSXRRUNTIME_SIMPLERENDER_H
