
#ifndef GSXRRUNTIME_SIMPLEMATH_H
#define GSXRRUNTIME_SIMPLEMATH_H
/**
 *  Created     :   2021-03-1
 *  Copyright (c)  CMCC Technologies, All rights reserved.
 */

#include <cmath>
namespace SIMPLE_MATH{
#define PI 3.14159265358979323846
#define PI_2 1.57079632679489661923
#define SF_GyroMeasError M_PI * (5.0f / 180.0f)
#define SF_GyroMeasDrift M_PI * (0.2f / 180.0f)
#define SF_Beta sqrt(3.0f / 4.0f) * SF_GyroMeasError
#define SF_Zeta sqrt(3.0f / 4.0f) * SF_GyroMeasDrift
#define SF_Kp 2.0f * 5.0f
#define SF_Ki 0.0f
#ifndef DEG2RAD
#define DEG2RAD (float)PI / 180.0f
#endif
#ifndef RAD2DEG
#define RAD2DEG (float)180.0f / PI
#endif
/**
 *
 * 定义一些用于渲染模块和交互模块使用的基本矢量结构和四元数数据结构，以及数学计算的相关辅助函数等。
 * 目前代码中只定义了满足最基本需求的Vector2f,Vector3f,Quaternion等数据类型，也可以使用比较成熟的
 * 数学库来维护该功能，例如：https://github.com/g-truc/glm
 * 涉及到的矩阵运算，矢量运算，四元数运算等，此处不做详细说明。
 *
 */

/**
 * 二维矢量数据结构体
 */
    typedef struct Vector2f{
        float x;
        float y;
        Vector2f():x(0.f),y(0.f){}
        Vector2f(float _x,float _y):x(_x),y(_y){}
        float GetMode() const {
            const float SMALLEST_NON_DENORMAL = 1.1754943508222875e-038f;
            const float rcp = (x >= SMALLEST_NON_DENORMAL) ? 1.0f / sqrtf(x*x+y*y) : 1.0f;
            return rcp;
        }
        inline Vector2f Normalized() const {float ret=GetMode();return Vector2f(x*ret,y*ret);}
        inline Vector2f operator*(float f)const {return Vector2f(x*f,y*f);}
    }Vector2f;
/**
 * 三维矢量数据结构体
 */
    typedef struct Vector3f{
        float x;
        float y;
        float z;
        Vector3f():x(0),y(0),z(0){}
        Vector3f(float _x,float _y,float _z):x(_x),y(_y),z(_z){}
        inline float GetModeSquared() const {return x*x+y*y+z*z;}
        inline float GetMode() const {return sqrt(x*x+y*y+z*z);}
        inline Vector3f Normalized() const {float ret=1/GetMode();return Vector3f(x*ret,y*ret,z*ret);}
    }Vector3f;
/**
 * 四元数数据结构体
 */
    typedef struct Quaternion{
        float x;
        float y;
        float z;
        float w;
        Quaternion():x(0.),y(0.),z(0.),w(1.){}
        Quaternion(float _x,float _y,float _z,float _w):x(_x),y(_y),z(_y),w(_w){}
        Quaternion(uint32_t axis,float angle,int32_t d=1,int32_t s=1){
            float half = s * d * sin(angle * 0.5);
            float v[3]={0,0,0};
            v[axis] = half;
            w = cos(angle * 0.5);
            x = v[0];
            y = v[1];
            z = v[2];
        }
        Quaternion(Vector3f axis,float angle){
            if(0==axis.GetModeSquared()){
                x=y=z=0;w=1;
                return;
            }
            Vector3f tempV= axis.Normalized();
            float halfSin= sin(angle*0.5);
            w=cos(angle*0.5);
            x = tempV.x * halfSin;
            y = tempV.y * halfSin;
            z = tempV.z * halfSin;
        }
        inline Quaternion MulQuat(Quaternion b) const{
            return Quaternion(w * b.x + x * b.w + y * b.z - z * b.y,
                              w * b.y - x * b.z + y * b.w + z * b.x,
                              w * b.z + x * b.y - y * b.x + z * b.w,
                              w * b.w - x * b.x - y * b.y - z * b.z);
        }
    }Quatf;
}
#endif //GSXRRUNTIME_SIMPLEMATH_H
